import unittest
import tempfile
import os
from slapos.libnetworkcache import NetworkcacheClient
from slapos.signature import parseArgument, \
                             createPrivateKeyAndCertificateFile

class OfflineTest(unittest.TestCase):
  def test_download_offline(self):
    nc = NetworkcacheClient('http://127.0.0.1:0', 'http://127.0.0.1:0')
    self.assertRaises(IOError, nc.download, 'sha512sum')

  def test_upload_offline(self):
    content = tempfile.TemporaryFile()
    nc = NetworkcacheClient('http://127.0.0.1:0', 'http://127.0.0.1:0')
    self.assertRaises(IOError, nc.upload, content)

class LibNetworkCacheMixin(unittest.TestCase):

  def setUp(self):
    ''' Setup the test. '''
    self.pub_file_descriptor = tempfile.NamedTemporaryFile()
    self.priv_file_descritor = tempfile.NamedTemporaryFile()
    self.signature_certificate_file = self.pub_file_descriptor.name
    self.signature_private_key_file = self.priv_file_descritor.name
    self.signature_creation_argument_list = \
        ('--signature-certificate-file', self.signature_certificate_file,
         '--signature-private-key-file', self.signature_private_key_file,
         '--country', 'BR',
         '--state-name', 'Campos',
         '--locality-name', 'Rio de Janeiro',
         '--organization-name', 'Nexedi',
         '--organization-unit-name', 'Dev',
         '--common-name', 'R500.com',
         '--email', 'test@example.com')

  def tearDown(self):
    ''' Remove the files which have been created during the test. '''
    self.priv_file_descritor.close()
    self.pub_file_descriptor.close()


class GenerateSignatureScriptTest(LibNetworkCacheMixin):
  ''' Class which must test the signature.py script. '''

  def test_parse_argument_with_empty_list(self):
    '''
      If the argument list is empty, then the parseArgument method should
    return a dictionary with default argument values.
    '''
    default_dict = {'organization_name': 'Default Company Ltd',
                    'state_name': 'Default Province',
                    'organization_unit_name': '',
                    'common_name': '',
                    'country': 'XX',
                    'locality_name': 'Default City',
                    'signature_private_key_file': 'private.pem',
                    'signature_certificate_file': 'public.pem',
                    'email': ''}
    self.assertEquals(default_dict, parseArgument())

  def test_parse_argument(self):
    '''
      Check if the argument is properly set.
    '''
    option_dict = parseArgument(*self.signature_creation_argument_list)
    size_argument_list = len(self.signature_creation_argument_list)/2
    size_option_dict = len(option_dict)
    self.assertEquals(size_argument_list, size_option_dict,
        "Argument list should have the same size of option dict.")

    # Assert if the values are equals.
    for value in option_dict.values():
      self.assertTrue(value in self.signature_creation_argument_list,\
          '%s is not in %s.' % (value, self.signature_creation_argument_list))

  def test_key_and_certificate_file_creation(self):
    '''
      Check if key file and the certificate file are being created correctly.
    '''
    option_dict = parseArgument(*self.signature_creation_argument_list)
    cert_as_text = createPrivateKeyAndCertificateFile(**option_dict)

    self.assertTrue(os.path.exists(self.signature_certificate_file))
    self.assertTrue(os.path.exists(self.signature_private_key_file))
